-- <Options> ------------------------------------------------------
local timerTextSize = 9     -- Font size of the countdown text.
local timerSpiral = true    -- Toggles the cooldown swipe.
local timerText = true      -- Toggles the cooldown countdown text.
local iconSize = 24         -- Size of the icons.
local MAX_UNITS = 3         -- Number of arena frames to attach to(max). Set to 5 if you play 5v5 arenas.
-------------------------------------------------------------------

local icons = {}
local ArenaGUID
local EventHandler

local SetPoint, SetTexture, UnitGUID, GetTime, GetSpellTexture, SetCooldown, SetBackdropBorderColor, GetSpecializationInfoByID, GetArenaOpponentSpec, band, CLOR_FRIENDLY, _ =
      SetPoint, SetTexture, UnitGUID, GetTime, GetSpellTexture, SetCooldown, SetBackdropBorderColor, GetSpecializationInfoByID, GetArenaOpponentSpec, bit.band, COMBATLOG_OBJECT_REACTION_FRIENDLY, _

-- Setup icons
do
    local categories = {"Stun", "Root", "Silence", "Incapacitate", "Disorient"}

	local function FrameOnShow(self)
		local posX = -24
		local unit = self.unit

		for i = 1, #categories do
			if icons[categories[i]..unit].shown then
				posX = posX - iconSize
			end
		end

		self:SetPoint("LEFT", posX, 22)
		self.shown = true
	end

	local function FrameOnHide(self)
		local posX = -24
		local unit = self.unit -- Avoid extra lookups
		local time = self.time
		self.shown = false
		self.applied = 0

		for i = 1, #categories do
			local f = icons[categories[i]..unit]
			local ftime = f.time

			if f.shown and ftime == time and ftime > (time - .001) or ftime > time then -- ftime and time cant be equal
				f:SetPoint("LEFT", posX, 22)
				posX = posX - iconSize
			end
		end
	end

	for i = 1, MAX_UNITS do
		local arenaFrame = _G["ArenaEnemyFrame"..i]

		for j = 1, #categories do
			local f = CreateFrame("Cooldown", nil, arenaFrame, "CooldownFrameTemplate")
			f:ClearAllPoints()
			f.time, f.applied, f.unit, f.shown = 0, 0, i, false
			f:SetHideCountdownNumbers(not timerText)
			f:SetSize(iconSize, iconSize)
			f:SetDrawSwipe(timerSpiral)
			f:SetSwipeColor(0, 0, 0, 0.60)
			f:SetDrawEdge(false)
			f:EnableMouse(false)
			f:SetFrameLevel(2)
			f:Hide()

			-- Texture
			f.icon = f:CreateTexture(nil, "BACKGROUND")
			f.icon:SetAllPoints(f)
			f:SetBackdrop({
				edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
				tileSize = iconSize + 4, edgeSize = 0.4 * iconSize,
			})

			-- Countdown
			for _, region in next, {f:GetRegions()} do
				if region:GetObjectType() == "FontString" then
					region:SetFont("Fonts\\FRIZQT__.TTF", timerTextSize, "OUTLINE")
					region:SetPoint("CENTER", f, 0, 0)
					break
				end
			end

			f:SetScript("OnShow", FrameOnShow)
			f:SetScript("OnHide", FrameOnHide)
			icons[categories[j]..i] = f
		end
	end
end

do
	local function PlayerIsHunterBM(i) -- Beast Mastery hunters shares DR with pet
		return GetSpecializationInfoByID(GetArenaOpponentSpec(i) or 0) == 253 -- bool
	end

	-- Memoize/cache for UnitGUIDs
	ArenaGUID = setmetatable({}, {
		__index = function(self, destGUID)
			local unitID = 0
			if not destGUID then return end

			for i = 1, MAX_UNITS do
				if UnitGUID("arena"..i) == destGUID then
					unitID = i
					break
				elseif UnitGUID("arenapet"..i) == destGUID then
					if PlayerIsHunterBM(i) then
						unitID = i
						break
					end
				end
			end

			self[destGUID] = unitID -- note: 0 will be added to blacklist
			return unitID
		end
	})
end

do
	local events = CreateFrame("Frame")
	local spellIDs = select(2, ...).spellList

	local function ActivateDR(spellID, category, unit)
		local f = icons[category..unit]
		if not f then -- Category is disabled
			spellIDs[spellID] = nil
			return
		end

		local time = GetTime()
		local applied = f.applied
		f.time = time
		f.applied = applied + 1
		f.icon:SetTexture(GetSpellTexture(spellID))
		f:SetBackdropBorderColor(1, applied == 2 and 1 or 0, 0, applied > 1 and 1 or 0)
		f:SetCooldown(time, 18)
	end

	function EventHandler(self, event, ...)
		if event == "COMBAT_LOG_EVENT_UNFILTERED" then
			local _, eventType, _, _, _, srcFlags, _, destGUID, _, _, _, spellID, _, _, auraType = ...

			if eventType == "SPELL_AURA_REMOVED" or eventType == "SPELL_AURA_REFRESH" and auraType == "DEBUFF" then
				if band(srcFlags, CLOR_FRIENDLY) > 0 then
					local category = spellIDs[spellID]
					local unitID = ArenaGUID[destGUID]

					if category and unitID and unitID > 0 then
						ActivateDR(spellID, category, unitID)
						return
					end
				end
			end
		else
			if select(2, IsInInstance()) == "arena" then
				wipe(ArenaGUID)
				self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			else
				self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
			end
		end
	end

	events:RegisterEvent("PLAYER_ENTERING_WORLD")
	events:SetScript("OnEvent", EventHandler)
	events:Hide()
end